% -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-
%               SAMPLE CODE FOR THE PERFORMANCE PREDICTION CHALLENGE
%             Isabelle Guyon -- isabelle@clopinet.com -- September 2005
% -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

% DISCLAIMER: ALL INFORMATION, SOFTWARE, DOCUMENTATION, AND DATA ARE PROVIDED "AS-IS" 
% ISABELLE GUYON AND/OR OTHER ORGANIZERS DISCLAIM ANY EXPRESSED OR IMPLIED WARRANTIES, 
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
% FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY OF NON-INFRIGEMENT OF ANY THIRD PARTY'S 
% INTELLECTUAL PROPERTY RIGHTS. IN NO EVENT SHALL ISABELLE GUYON AND/OR OTHER ORGANIZERS 
% BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF SOFTWARE, DOCUMENTS, 
% MATERIALS, PUBLICATIONS, OR INFORMATION MADE AVAILABLE FOR THE CHALLENGE. 
%
% ---------
% Tested and modified by: 
% Gideon Dror, gideon@mta.ac.il
% Amir Reza Saffari Azar, amir@ymer.org.
% May 2006
% ---------

%% Initialization
clear all
close all
clc

% -o-|-o-|-o-|-o-|-o-|-o-|-o- BEGIN USER-PREFERENCES -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

% 1) User-defined directories (no slash at the end of the names):
% --------------------------------------------------------------
% The present set up supposes that you are now in the directory sample_code
% and you have the following directory tree, where Challenge is you project directory:
% Challenge/Data
% Challenge/Results
% Challenge/Zipped
% Challenge/Models
% Challenge/Clop
% Challenge/Clop/sample_code

my_root     = '../..';              % Change that to the directory of your project

data_dir    = [my_root '/Data'];    % Path to the five data directories downloaded (ADA, GINA, HIVA, NOVA, SYLVA).
resu_dir    = [my_root '/Results']; % Where the results will end up.    
zip_dir     = [my_root '/Zipped'];  % Zipped files with results ready to go!
model_dir   = [my_root '/Models'];  % Where the trained models will end up.
                                    % Note: you have to include the models in
                                    % your submissions for a valid submission.
                                    % In this example, the models are save in full
                                    % in the model_dir directory and a lean
                                    % version (with the hyperparameters and
                                    % the smallest trained entities) is
                                    % save in the resu_dir directory.
% If you just downloaded the sample code, you do not have the spider
% and Challenge Learning Object Package (CLOP). 
% You may download them from: 
% http://clopinet.com/isabelle/Projects/modelselect/Clop.zip. 
% But you do not need to, you can point the code_dir to the sample_code
% directory:
code_dir    = [my_root '/Clop'];    % Path to the sample code or the 
                                    % Challenge Learning Objects Package (CLOP).
score_dir   = [my_root '/Score'];   % Directory where the model scores are found.

ForceOverWrite  = 1;                % Change this value to 0 if you want to be warned when
                                    % a file already exists before saving a result or model.
                                    
DoNotLoadTestData   = 1;            % To save memory, does not load the test data

MergeDataSets   = 0;                % If this flag is zero, training is done on the
                                    % training data only. Otherwise training
                                    % and validation data are merged. 

FoldNum = 0;                        % If this flag is positive, 
                                    % k-fold cross-validation is performed.
                                    % with k=FoldNum.

% 2) Choose your data and models
% ------------------------------
% dataset     = {'ada', 'gina', 'hiva', 'nova', 'sylva'}; 
dataset     = {'ada'}; 
modelset    = {'Relief+neuralNet'};
                                    % This should be an array of model example names
                                    % e.g. {'zarbi', 'Prepro+linearSVC'}
                                    % If you leave this array is empty, it will 
                                    % be replaced by a default list of model names.                              
                                    % For a list of model examples, use:
                                    % > model_examples

% -o-|-o-|-o-|-o-|-o-|-o-|-o- END USER-PREFERENCES -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-
fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n');

% Set the path and defaults properly; create directories
% ------------------------------------------------------
addpath(code_dir);
if exist('use_spider_clop.m') == 2, 
    use_spider_clop(code_dir);
elseif exist('code_version.m') == 2, 
    fprintf('Sample code version : %s\n', code_version('sample_code'));
else disp 'ERROR: Wrong code path. Check your directories and path variables.';
    if exist('README.txt') == 2, type README.txt; end
    if exist('Data/README.txt') == 2, type Data/README.txt; end
    if exist('Clop/README.txt') == 2, type Clop/README.txt; end
    return; 
end
if isempty(modelset), modelset =  model_examples; end
makedir(resu_dir);
makedir(zip_dir);
makedir(model_dir);
makedir(score_dir);

%% Train/Test
% LOOP OVER DATASETS 
% ===================
for k = 1:length(dataset)
    
	data_name   = dataset{k};
    
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-      %s      -o-|-o-|-o-|-o-|-o-|-o-\n', upper(data_name));
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n\n');
    %disp(data_dir)
    % Create a data structure and check the data
    %===========================================
    fprintf('-- %s loading data --\n', upper(data_name));
    input_dir   = [data_dir '/' upper(data_name)];
    input_name  = [input_dir '/' data_name];
    
    D   = create_data_struct(input_name, DoNotLoadTestData);
    % Each member of D (D.train, D.valid, and D.test) 
    % is a data object (a structure) with members X and Y (if Y is given). 
    
    % New: compute data statistics
    data_stats(D);
    
    fprintf('-- %s data loaded --\n', upper(data_name));
    % Note: the data are saved as a Matlab structure 
    % so they will load faster the second time around. 

    % LOOP OVER MODELS 
    % ================
    for j = 1:length(modelset)
        
        model_name  = modelset{j};
        
        resu_name   = [resu_dir '/' model_name ];
        makedir(resu_name);
        resu_name   = [resu_name '/' data_name];
        
        % Build a model
        %==============
        fprintf('-- %s-%s building  model\n', upper(data_name), upper(model_name));
        % "model_examples" calls a model constructor and returns a learning object.
        % Enter at the prompt "> type model_examples" to view the examples.
        % All learning objects have the two methods "train" and "test".
        % To see the data members, type at the prompt "> struct(my_model)".
        my_model    = model_examples(model_name, data_name);
        % Save the model (untrained) in the result file
        save_model([resu_name , '_model'], my_model, ForceOverWrite, 0);
        % Reload it just to make sure
        my_model    = load_model([resu_name , '_model']);
        fprintf('-- %s-%s model built\n', upper(data_name), upper(model_name));
        
        % Train the model
        %================
        if MergeDataSets
            % Get rid of the validation set
            D.train = data([D.train.X ; D.valid.X], [D.train.Y ; D.valid.Y]);
            rmfield(D, 'valid');
        end
        
        if FoldNum > 0
            % Create a CV model
            cv_model    = cv(my_model, {['folds=' num2str(FoldNum)], 'store_all=0'});
            fprintf('-- %s-%s performing %d fold cross-validation\n', upper(data_name), upper(model_name), FoldNum);
            % Call the method "train" of the object "cv_model":
            cv_output   = train(cv_model, D.train); 
            % Collect the results
            OutX = []; OutY = []; ber =[];
            for kk = 1:FoldNum,
                outX    = cv_output.child{kk}.X;
                outY    = cv_output.child{kk}.Y;
                OutX    = [OutX; outX]; 
                OutY    = [OutY; outY]; 
                ber(kk) = balanced_errate(outX, outY);
            end
            cvber   = balanced_errate(OutX, OutY);
            ebar    = std(ber,1)/sqrt(FoldNum);
            fprintf('CV BER=%5.2f+-%5.2f%%\n', 100*cvber, 100*ebar); 
            fprintf('-- %s-%s cross-validation done in %5.2f seconds\n', upper(data_name), upper(model_name), toc);
            fprintf('-- %s-%s training model on all training data\n', upper(data_name), upper(model_name));
        end
        
        fprintf('-- %s-%s training model\n', upper(data_name), upper(model_name));
        tic;
        [train_output, my_model]    = train(my_model, D.train);  
        [ber, e1, e2, sigma]        = balanced_errate(train_output.X,train_output.Y);
        fprintf('TRAIN BER=%5.2f +-%5.2f%%\n', 100*ber, 100*sigma); 
        fprintf('-- %s-%s model trained in %5.2f seconds\n', upper(data_name), upper(model_name), toc);
        
        guessed_ber = balanced_errate(train_output.X,train_output.Y); % Rough guess: the training error!
        fprintf('Making the very tentative BER guess of %5.2f%%\n', 100*guessed_ber);

        % Save the trained model
        %=======================
        fprintf('-- %s-%s saving the model\n', upper(data_name), upper(model_name));
        mresu_name  = [model_dir '/' model_name];
        makedir(mresu_name);
        mresu_name  = [mresu_name '/' data_name '_model'];
        save_model(mresu_name, my_model, ForceOverWrite, 0);
        % Reload it just to make sure
        my_model    = load_model(mresu_name);
        fprintf('-- %s-%s models saved as %s*\n', upper(data_name), upper(model_name), mresu_name);
        
        % Test the model
        %===============
        if DoNotLoadTestData | MergeDataSets
            % Now (re-)load everything
            D = create_data_struct(input_name);
        end
        % Note: in order to be able to make valid challenge
        % submissions, here we keep the original data split.
        % Thus the training error will differ from the
        % error rate on all the training data and the validation
        % error will be similar to the training error (since the
        % validation data was used for training.)
        tic;
        fprintf('-- %s-%s testing model\n', upper(data_name), upper(model_name));
        set_name    = fieldnames(D); % 'train', 'valid' and 'test'
        
        for i = 1:length(set_name) % Loop over training, validation, and test set
            % Call the method "test" of the object "my_model" 
            % on D.train, D.valid, or D.test:
            output  = test(my_model, D.(set_name{i})); 
            discriminant{i} = output.X; % The "test" method returns the predicted discriminant values in output.X
            target = output.Y; % If the targets were provided in the dataset, they are copied in output.Y
            if ~isempty(target)
                [ber, e1, e2, sigma] = balanced_errate(discriminant{i},target);
                guess_error = challenge_error(guessed_ber, ber);
                score_type = 3;
                score = challenge_error(guessed_ber, ber, score_type, sigma);
                fprintf('%s, BER=%5.2f +-%5.2f%%, guess_error=%5.2f%%, score=%5.2f%%\n', ...
                    set_name{i}, 100*ber, 100*sigma, 100*guess_error, 100*score); 
            end
        end
        fprintf('-- %s-%s model tested in %5.2f seconds\n', upper(data_name), upper(model_name), toc);  
        
        % Save the results
        %=================
        fprintf('-- %s-%s saving the results\n', upper(data_name), upper(model_name));
        for i=1:length(set_name) % Loop over 'train', 'valid', and 'test'
            save_outputs([[resu_name '_' set_name{i}] '.resu'], sign(discriminant{i}), ForceOverWrite);        
            save_outputs([[resu_name '_' set_name{i}] '.conf'], abs(discriminant{i}), ForceOverWrite);        
        end
        
        save_outputs([resu_name '.guess'], guessed_ber, ForceOverWrite);
        fprintf('-- %s-%s results saved as %s*\n', upper(data_name), upper(model_name), resu_name);
        
        fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n');
        
    end % Loop over models
end % Loop over datasets

%% Zip the archives so they are ready to go!
%%-----------------------------------------
if ~usejava('jvm'), warning('Java is not loaded, failed to generate ZIP files !!!'); return; end
for k = 1:length(modelset)
    model_name  = modelset{k};
    zip_name    = zipall(model_name, resu_dir, zip_dir);
    if ~isempty(zip_name)
        fprintf('-- %s zip archive created, see %s --\n', upper(model_name), zip_name);
    end
end

% Score the models
fprintf('-- %s scoring the models --\n');
simple_score(resu_dir, data_dir, score_dir);